"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("@umijs/utils");
const assert_1 = __importDefault(require("assert"));
const path_1 = __importDefault(require("path"));
const utils_2 = require("../../utils");
const utils_3 = require("../utils");
const webpackBundler = (0, utils_1.importLazy)(path_1.default.dirname(require.resolve('@umijs/bundler-webpack/package.json')));
const makoBundler = (0, utils_1.importLazy)(path_1.default.dirname(require.resolve(process.env.OKAM || '@umijs/bundler-mako/package.json')));
const { CSSMinifier, JSMinifier, } = (0, utils_1.importLazy)(require.resolve('@umijs/bundler-webpack/dist/types'));
const extensions = ['.js', '.jsx', '.ts', '.tsx', '.cjs', '.mjs'];
async function bundle(opts) {
    const enableCache = process.env.FATHER_CACHE !== 'none';
    const closeHandlers = [];
    for (let i = 0; i < opts.configProvider.configs.length; i += 1) {
        const config = opts.configProvider.configs[i];
        const { plugins: extraPostCSSPlugins, ...postcssLoader } = config.postcssOptions || {};
        const babelSCOpts = (0, utils_3.getBabelStyledComponentsOpts)(opts.configProvider.pkg);
        // workaround for combine continuous onBuildComplete log in watch mode
        const logStatus = utils_1.lodash.debounce(() => utils_2.logger.info(`Bundle from ${utils_1.chalk.yellow(config.entry)} to ${utils_1.chalk.yellow(path_1.default.join(config.output.path, config.output.filename))}`), 100, { leading: true, trailing: false });
        // log for normal build
        !opts.watch && logStatus();
        const options = {
            cwd: opts.cwd,
            hmr: false,
            watch: opts.watch,
            devServer: false,
            config: {
                alias: config.alias,
                autoprefixer: config.autoprefixer,
                chainWebpack: config.chainWebpack,
                stats: false,
                define: config.define,
                devtool: !!config.sourcemap && 'source-map',
                externals: config.externals,
                outputPath: config.output.path,
                // postcss config
                extraPostCSSPlugins,
                postcssLoader,
                clean: false,
                ...(config.extractCSS !== false ? {} : { styleLoader: {} }),
                // ref: https://github.com/umijs/mako/issues/1062
                inlineCSS: config.extractCSS !== false ? false : { 'TODO: REMOVE ME': 1 },
                // less config
                theme: config.theme,
                // compatible with IE11 by default
                targets: (0, utils_3.getBundleTargets)(config),
                jsMinifier: JSMinifier.terser,
                cssMinifier: CSSMinifier.cssnano,
                extraBabelIncludes: [/node_modules/],
                // set cache parent directory, will join it with `bundler-webpack`
                // ref: https://github.com/umijs/umi/blob/8dad8c5af0197cd62db11f4b4c85d6bc1db57db1/packages/bundler-webpack/src/build.ts#L32
                cacheDirectoryPath: (0, utils_2.getCachePath)(),
            },
            entry: {
                [path_1.default.parse(config.output.filename).name]: path_1.default.join(opts.cwd, config.entry),
            },
            babelPreset: [
                require.resolve('@umijs/babel-preset-umi'),
                {
                    presetEnv: {
                        targets: (0, utils_3.getBundleTargets)(config),
                    },
                    presetReact: (0, utils_3.getBabelPresetReactOpts)(opts.configProvider.pkg, opts.cwd),
                    presetTypeScript: {},
                    pluginTransformRuntime: {},
                    pluginLockCoreJS: {},
                    pluginDynamicImportNode: false,
                },
            ],
            beforeBabelPlugins: [
                require.resolve('babel-plugin-dynamic-import-node'),
                ...(babelSCOpts
                    ? [[require.resolve('babel-plugin-styled-components'), babelSCOpts]]
                    : []),
            ],
            extraBabelPresets: config.extraBabelPresets,
            extraBabelPlugins: config.extraBabelPlugins,
            // configure library related options
            chainWebpack(memo) {
                memo.output.libraryTarget('umd');
                if (config.bundler === 'mako') {
                    (0, assert_1.default)(config.name, `Mako bundler need set name in umd config`);
                }
                if (config === null || config === void 0 ? void 0 : config.name) {
                    memo.output.library(config.name);
                }
                // modify webpack target
                if (config.platform === 'node') {
                    memo.target('node');
                }
                if (enableCache) {
                    // use father version as cache version
                    memo.merge({
                        cache: { version: require('../../../package.json').version },
                    });
                }
                // also bundle svg as asset, because father force disable svgr
                const imgRule = memo.module.rule('asset').oneOf('image');
                if (imgRule.get('test')) {
                    imgRule.test(new RegExp(imgRule.get('test').source.replace(/(\|png)/, '$1|svg')));
                }
                // disable progress bar
                memo.plugins.delete('progress-plugin');
                // auto bump analyze port
                /* istanbul ignore if -- @preserve */
                if (process.env.ANALYZE) {
                    memo.plugin('webpack-bundle-analyzer').tap((args) => {
                        args[0].analyzerPort += i;
                        return args;
                    });
                }
                return memo;
            },
            // enable webpack persistent cache
            ...(enableCache
                ? {
                    cache: {
                        buildDependencies: opts.buildDependencies,
                    },
                }
                : {}),
            // collect close handlers for watch mode
            onBuildComplete({ isFirstCompile, close }) {
                if (isFirstCompile)
                    closeHandlers.push(close);
                // log for watch mode
                else
                    logStatus();
            },
            onDevCompileDone() { },
            disableCopy: true,
        };
        if (config.bundler === 'mako') {
            require('@umijs/bundler-webpack/dist/requireHook');
            const entry = (0, utils_1.tryPaths)(extensions.map((ext) => path_1.default.join(opts.cwd, `${config.entry}${ext}`)));
            (0, assert_1.default)(entry, `Cannot find entry file ${config.entry}`);
            options.entry = {
                [path_1.default.parse(config.output.filename).name]: entry,
            };
            await makoBundler.build(options);
        }
        else {
            await webpackBundler.build(options);
        }
    }
    // return watching closer for watch mode
    if (opts.watch) {
        return {
            close() {
                return Promise.all(closeHandlers.map((handler) => new Promise((resolve) => handler(resolve))));
            },
        };
    }
}
exports.default = bundle;
